unit deBoldFormSaverValidator;

{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: deBoldFormSaverValidator.PAS, released November 28, 2002.

The Initial Developer of the Original Code is Peter Morris (pete@stuckindoors.com),
Portions created by Peter Morris are Copyright (C) 2002 Peter Morris.
All Rights Reserved.

Purpose of file:
Validates all dirty objects in a BoldFormSaver to ensure that it is valid to
call BoldFormSaver.OK

Contributor(s):
<None>

Last Modified: November 28, 2002.

You may retrieve the latest version of this file at
located at http://www.droopyeyes.com
-----------------------------------------------------------------------------}
//MODIFICATIONS
//=============
//Date: December 20, 2002
//By: Peter Morris
//Changes: 
//  Altered the technique used to validate a newly added object in order to 
//    avoid an index out of bounds error.       
//=============


interface
uses
  Classes, SysUtils, BoldSystem, BoldSubscription, BoldElements, BoldAttributes,
  BoldHandles, BoldFormSaver;

type
  TdeCustomBoldFormSaverValidator = class;
  TdeBoldFormSaverValidator = class;
  TFormSaverValidationEvent = procedure (Sender: TObject; Valid: Boolean) of 
          object;
  TdeCustomBoldFormSaverValidator = class (TComponent)
  private
    FBoldFormSaver: TBoldFormSaver;
    FOnValidation: TFormSaverValidationEvent;
    FSubscriber: TBoldPassthroughSubscriber;
    procedure SetBoldFormSaver(Value: TBoldFormSaver);
  protected
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); 
            override;
    procedure ReceiveEvent(Originator: TObject; OriginalEvent: TBoldEvent; 
            RequestedEvent: TBoldRequestedEvent); virtual;
    procedure SubscribeToFormSaver;
    procedure SubscribeToObjects;
    property BoldFormSaver: TBoldFormSaver read FBoldFormSaver write 
            SetBoldFormSaver;
    property OnValidation: TFormSaverValidationEvent read FOnValidation write 
            FOnValidation;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;
  
  TdeBoldFormSaverValidator = class (TdeCustomBoldFormSaverValidator)
  published
    property BoldFormSaver;
    property OnValidation;
  end;
  

implementation

{
*********************** TdeCustomBoldFormSaverValidator ************************
}
constructor TdeCustomBoldFormSaverValidator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FSubscriber := TBoldPassthroughSubscriber.Create(ReceiveEvent)
end;

destructor TdeCustomBoldFormSaverValidator.Destroy;
begin
  FreeAndNil(FSubscriber);
  inherited Destroy;
end;

procedure TdeCustomBoldFormSaverValidator.Loaded;
begin
  SubscribeToFormSaver;
end;

procedure TdeCustomBoldFormSaverValidator.Notification(AComponent: TComponent; 
        Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = BoldFormSaver) then
    BoldFormSaver := nil;
end;

procedure TdeCustomBoldFormSaverValidator.ReceiveEvent(Originator: TObject; 
        OriginalEvent: TBoldEvent; RequestedEvent: TBoldRequestedEvent);
var
  I: Integer;
  Valid: Boolean;
  Element: TBoldIndirectElement;
begin
  Valid := True;
  if (OriginalEvent = beItemAdded) and (RequestedEvent = beItemAdded) then
    with BoldFormSaver.DirtyObjects do
      BoldObjects[Count - 1].SubscribeToExpression('constraints->select(c|not c)->size = 0',
        FSubscriber, True, False);
  Element := TBoldIndirectElement.Create;
  try
    for I := BoldFormSaver.DirtyObjects.Count - 1 downto 0 do
    begin
      BoldFormSaver.DirtyObjects.BoldObjects[I].EvaluateExpression('constraints->select(c|not c)->size = 0', Element);
      if not (Element.Value as TBABoolean).AsBoolean then
      begin
        Valid := False;
        Break;
      end;
    end;
  finally
    Element.Free;
  end;
  if Assigned(OnValidation) then
    OnValidation(Self, Valid);
end;

procedure TdeCustomBoldFormSaverValidator.SetBoldFormSaver(Value: 
        TBoldFormSaver);
begin
  if FBoldFormSaver = Value then Exit;
  FSubscriber.CancelAllSubscriptions;
  FBoldFormSaver := Value;
  if ComponentState * [csLoading, csDesigning] = [] then
    if Assigned(FBoldFormSaver) then SubscribeToFormSaver;
end;

procedure TdeCustomBoldFormSaverValidator.SubscribeToFormSaver;
begin
  if Assigned(BoldFormSaver) then
  begin
    BoldFormSaver.FreeNotification(Self);
    BoldFormSaver.DirtyObjects.AddSubscription(FSubscriber, beItemAdded, beItemAdded);
    BoldFormSaver.DirtyObjects.AddSubscription(FSubscriber, beValueChanged, beValueChanged);
    BoldFormSaver.DirtyObjects.AddSubscription(FSubscriber, beItemDeleted, beItemDeleted);
    BoldFormSaver.DirtyObjects.AddSubscription(FSubscriber, beItemReplaced, beItemReplaced);
    SubscribeToObjects;
  end;
end;

procedure TdeCustomBoldFormSaverValidator.SubscribeToObjects;
var
  I: Integer;
begin
  if Assigned(BoldFormSaver) then
    with BoldFormSaver.DirtyObjects do
      for I := 0 to Count - 1 do
        BoldObjects[I].SubscribeToExpression('constraints->select(c|not c)->size = 0',
          FSubscriber, True, False);
end;


end.
