unit deBoldObjectValidator;

{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: deBoldObjectValidator.PAS, released July 23, 2002.

The Initial Developer of the Original Code is Peter Morris (pete@stuckindoors.com),
Portions created by Peter Morris are Copyright (C) 2002 Peter Morris.
All Rights Reserved.

Purpose of file:
Validates objects based on their constraints.

Contributor(s):
Eric Holton

Last Modified: December 3, 2002.

You may retrieve the latest version of this file at
located at http://www.droopyeyes.com
-----------------------------------------------------------------------------}
//MODIFICATIONS
//=============
//Date: December 3, 2002
//By: Peter Morris
//Changes: 
//  Added "Errors: TStrings" public property to TdeCustomBoldObjectValidator.
//  Added a better subscription method, AddSubscriptions is now virtual.       
//=============
//Date: December 9, 2002
//By: Peter Morris
//Changes: 
//  Subscribed to Constraints->Select(c| not c)->Size = 0 instead of
//  Constraints->Select(c not c)
//=============
//Date: December 18, 2002
//By: Peter Morris
//Changes: 
//  ValidateObject is called even if the object is nil so that the GUI is not
//    left showing errors for the previous object.
//  OnStartValidate and OnValidateComplete are also called if the object is nil.

interface
uses
  Classes, SysUtils, BoldSystem, BoldSubscription, BoldElements, BoldAttributes,
  BoldHandles;

type
  TValidationErrorEvent = procedure (Sender: TObject; ConstraintMessage: 
          string) of object;
  TdeCustomBoldObjectValidator = class (TComponent)
  private
    FBoldObject: TBoldObject;
    FErrors: TStrings;
    FObjectSubscriber: TBoldPassthroughSubscriber;
    FOnStartValidate: TNotifyEvent;
    FOnValidateComplete: TNotifyEvent;
    FOnValidationError: TValidationErrorEvent;
    function GetErrorCount: Integer;
    procedure SetBoldObject(const Value: TBoldObject);
  protected
    procedure AddSubscriptions; virtual;
    procedure ReceiveEvent(Originator: TObject; OriginalEvent: TBoldEvent; 
            RequestedEvent: TBoldRequestedEvent); virtual;
    procedure ValidateObject; virtual;
    property BoldObject: TBoldObject read FBoldObject write SetBoldObject;
    property OnStartValidate: TNotifyEvent read FOnStartValidate write 
            FOnStartValidate;
    property OnValidateComplete: TNotifyEvent read FOnValidateComplete write 
            FOnValidateComplete;
    property OnValidationError: TValidationErrorEvent read FOnValidationError 
            write FOnValidationError;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property ErrorCount: Integer read GetErrorCount;
    property Errors: TStrings read FErrors;
  end;
  
  TdeBoldObjectValidator = class (TdeCustomBoldObjectValidator)
  public
    property BoldObject;
  published
    property OnStartValidate;
    property OnValidateComplete;
    property OnValidationError;
  end;
  
  TdeCustomBoldHandleValidator = class (TdeCustomBoldObjectValidator)
  private
    FRootHandle: TBoldElementHandle;
    procedure SetRootHandle(const Value: TBoldElementHandle);
  protected
    procedure AddSubscriptions; override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); 
            override;
    procedure ReceiveEvent(Originator: TObject; OriginalEvent: TBoldEvent; 
            RequestedEvent: TBoldRequestedEvent); override;
    property RootHandle: TBoldElementHandle read FRootHandle write 
            SetRootHandle;
  public
    destructor Destroy; override;
  end;
  
  TdeBoldHandleValidator = class (TdeCustomBoldHandleValidator)
  published
    property OnStartValidate;
    property OnValidateComplete;
    property OnValidationError;
    property RootHandle;
  end;
  


implementation


const
  cRemoveReference = 1;
  cReValidate = 2;
  cHandleObjectChanged = 3;

{
************************* TdeCustomBoldObjectValidator *************************
}
constructor TdeCustomBoldObjectValidator.Create(AOwner: TComponent);
begin
  inherited;
  FErrors := TStringList.Create;
  FObjectSubscriber := TBoldPassthroughSubscriber.Create(ReceiveEvent);
end;

destructor TdeCustomBoldObjectValidator.Destroy;
begin
  FreeAndNil(FObjectSubscriber);
  FreeAndNil(FErrors);
  inherited;
end;

procedure TdeCustomBoldObjectValidator.AddSubscriptions;
begin
  FObjectSubscriber.CancelAllSubscriptions;
  if Assigned(BoldObject) then
  begin
    BoldObject.SubscribeToExpression('constraints->select(c|not c)',
      FObjectSubscriber, True, False);
    BoldObject.AddSmallSubscription(FObjectSubscriber, [beValueIdentityChanged],
      cHandleObjectChanged);
  end;
end;

function TdeCustomBoldObjectValidator.GetErrorCount: Integer;
begin
  Result := FErrors.Count;
end;

procedure TdeCustomBoldObjectValidator.ReceiveEvent(Originator: TObject; 
        OriginalEvent: TBoldEvent; RequestedEvent: TBoldRequestedEvent);
begin
  case RequestedEvent of
    cRemoveReference: BoldObject := nil;
    cReValidate: ValidateObject;
    breResubscribe: ValidateObject;
  end;
end;

procedure TdeCustomBoldObjectValidator.SetBoldObject(const Value: TBoldObject);
begin
  if Value <> BoldObject then
  begin
    FBoldObject := Value;
    AddSubscriptions;
    ValidateObject;
  end;
end;

procedure TdeCustomBoldObjectValidator.ValidateObject;
var
  I: Integer;
  Element: TBoldIndirectElement;
begin
  if Assigned(FOnStartValidate) then FOnStartValidate(Self);
  try
    FErrors.Clear;
    if Assigned(BoldObject) then
    begin
      Element := TBoldIndirectElement.Create;
      try
        BoldObject.EvaluateExpression('constraints->select(c|not c)', Element);
        with (Element.Value as TBoldList) do
        begin
          for I := 0 to Count - 1 do
          begin
            FErrors.Add((Elements[I] as TBAConstraint).Constraint.ModelName);
            if Assigned(FOnValidationError) then
              FOnValidationError(Self, (Elements[I] as TBAConstraint).Constraint.ModelName);
          end;
        end;
      finally
        Element.Free;
      end;
    end;
  finally
    if Assigned(FOnValidateComplete) then FOnValidateComplete(Self);
  end;
end;

{ TdeCustomBoldHandleValidator }

{
************************* TdeCustomBoldHandleValidator *************************
}
destructor TdeCustomBoldHandleValidator.Destroy;
begin
  RootHandle := nil;
  inherited;
end;

procedure TdeCustomBoldHandleValidator.AddSubscriptions;
begin
  inherited;
  if Assigned(RootHandle) then
  begin
    RootHandle.FreeNotification(Self);
    RootHandle.AddSmallSubscription(FObjectSubscriber, [beValueIdentityChanged], cHandleObjectChanged);
  end;
end;

procedure TdeCustomBoldHandleValidator.Loaded;
var
  RHandle: TBoldElementHandle;
begin
  inherited;
  RHandle := FRootHandle;
  RootHandle := nil;
  RootHandle := RHandle;
end;

procedure TdeCustomBoldHandleValidator.Notification(AComponent: TComponent; 
        Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then
    if AComponent = RootHandle then RootHandle := nil;
end;

procedure TdeCustomBoldHandleValidator.ReceiveEvent(Originator: TObject; 
        OriginalEvent: TBoldEvent; RequestedEvent: TBoldRequestedEvent);
begin
  if RequestedEvent = cHandleObjectChanged then
  begin
    if Assigned(RootHandle) and (RootHandle.Value is TBoldObject) then
      BoldObject := (RootHandle.Value as TBoldObject)
    else
      BoldObject := nil;
  end else
    inherited;
end;

procedure TdeCustomBoldHandleValidator.SetRootHandle(const Value: 
        TBoldElementHandle);
begin
  if Value <> FRootHandle then
  begin
    FRootHandle := Value;
    AddSubscriptions;
    if Assigned(RootHandle) and (RootHandle.Value is TBoldObject) then
      BoldObject := (RootHandle.Value as TBoldObject)
    else
      BoldObject := nil;
  end;
end;

end.
